
#include "m_bma250e.h"

const nrf_drv_spi_t   m_spi_master_0 = NRF_DRV_SPI_INSTANCE(0);


static volatile bool m_transfer_completed = true;
void spi_master_0_event_handler(nrf_drv_spi_event_t event)
{
    if(event == NRF_DRV_SPI_EVENT_DONE ){
       m_transfer_completed = true;
    }
}

void spi_master_init(void)
{
    uint32_t err_code = NRF_SUCCESS;
    nrf_drv_spi_config_t     config;

    config.irq_priority = APP_IRQ_PRIORITY_HIGH;//Same Bugs from Nordic...Shit!!
    config.orc          = 0xFF;
    config.frequency    = NRF_DRV_SPI_FREQ_125K;//reduce the speed because the board is not stable 2016/08/13
    config.ss_pin       = SPIM0_SS_PIN;
    config.sck_pin      = SPIM0_SCK_PIN;
    config.mosi_pin     = SPIM0_MOSI_PIN;
    config.miso_pin     = SPIM0_MISO_PIN;
    config.mode         = NRF_DRV_SPI_MODE_3;
    config.bit_order    = NRF_DRV_SPI_BIT_ORDER_MSB_FIRST;
	  
    err_code = nrf_drv_spi_init( &m_spi_master_0, &config, spi_master_0_event_handler);
    APP_ERROR_CHECK(err_code);
}

void MySPI_Write(uint8_t  SPI_TX[])
{
	 uint8_t  SPI_RX[2];
	 m_transfer_completed = false;
   uint32_t err_code =nrf_drv_spi_transfer(&m_spi_master_0, SPI_TX, 2, SPI_RX, 0);
	 APP_ERROR_CHECK(err_code);
	 while(m_transfer_completed==false);
}

void BMA250E_Runtime_Mode(void)
{
	 uint8_t  tx_buf[2];
	 //Reset
	 tx_buf[0] = 0x14; tx_buf[1] = 0xB6;
	 MySPI_Write(tx_buf);
	 nrf_delay_ms(10);
	 /*
    0x00;// 64ms /7.81 HZ
    0x09;// 32ms /15.63 HZ
	  0x0A;// 16ms /31.25 HZ
	  0x0B;// 8ms  /62.5 HZ
	  0x0C;// 4ms  /125 HZ
	  0x0D;// 2ms  /250 HZ
 	  0x0E;// 1ms  /500 HZ
    0x1F;// 0.5ms/1K HZ
   */
	 //PMU_BW
	 tx_buf[0] = 0x10; tx_buf[1] = 0x0C;//20151225,Fix this 0x0C Setting do not change!!
	 MySPI_Write(tx_buf);
	
	  //low power mode 0x12
	 tx_buf[0] = 0x12; tx_buf[1] = 0x00;
	 MySPI_Write(tx_buf);
	
	 //low power mode 0x11
	 tx_buf[0] = 0x11; tx_buf[1] = 0x00;
	 MySPI_Write(tx_buf);
	
	 //Clear Any Motion slop_en WakeUp MCU
	 tx_buf[0] = 0x16; tx_buf[1] = 0x00;
	 MySPI_Write(tx_buf);
	 
	 //set INT1 and INT2 Low Active
	 tx_buf[0] = 0x20; tx_buf[1] = 0x00;
	 MySPI_Write(tx_buf);
	
	 //clear INT1 as Output
	 tx_buf[0] = 0x19; tx_buf[1] = 0x00;
	 MySPI_Write(tx_buf);
	 
	 //PMU_RANGE1
	 tx_buf[0] = 0x0F; tx_buf[1] = 0x0C;//+-16G
	 MySPI_Write(tx_buf);
}

void BMA250E_Wakeup_Mode(void)
{
	 uint8_t  tx_buf[2];
	 //Reset
	 tx_buf[0] = 0x14; tx_buf[1] = 0xB6;
	 MySPI_Write(tx_buf);
	 nrf_delay_ms(10);
	
	 //PMU_BW
	 /*
    0x00 -- 64ms /7.81 HZ
    0x09 -- 32ms /15.63 HZ
	  0x0A -- 16ms /31.25 HZ
	  0x0B -- 8ms  /62.5 HZ
	  0x0C -- 4ms  /125 HZ
	  0x0D -- 2ms  /250 HZ
 	  0x0E -- 1ms  /500 HZ
    0x1F -- 0.5ms/1K HZ
    */
	 tx_buf[0] = 0x10; tx_buf[1] = 0x00;// 64ms/7.81 HZ 
	 MySPI_Write(tx_buf);
	
	 //PMU_RANGE1
	 tx_buf[0] = 0x0F; tx_buf[1] = 0x03;//+-2G
	 MySPI_Write(tx_buf);
	
	 //set INT1 and INT2 Low Active
	 tx_buf[0] = 0x20; tx_buf[1] = 0x00;
	 MySPI_Write(tx_buf);
	
	 //set INT1 as Output
	 tx_buf[0] = 0x19; tx_buf[1] = 0x04;
	 MySPI_Write(tx_buf);
	 
	 //set Any Motion slop_en WakeUp MCU
	 tx_buf[0] = 0x16; tx_buf[1] = 0x07;
	 MySPI_Write(tx_buf);
	
	 //low power mode 0x12
	 tx_buf[0] = 0x12; tx_buf[1] = 0x00;
	 MySPI_Write(tx_buf);
	
	 //low power mode 0x11
	 tx_buf[0] = 0x11; 
	 //tx_buf[1] = 0x40;//sp time 0.5ms
	 //tx_buf[1] = 0x47;//sp time 2ms
	 //tx_buf[1] = 0x52;//sp time 6ms
	 //tx_buf[1] = 0x5A;//sp time 100ms
	 //tx_buf[1] = 0x5C;//sp time 500ms
	 tx_buf[1] = 0x5E;//sp time 1S
	 MySPI_Write(tx_buf);
}

void ReadAccX16(m_bma250e_t* m_g)
{ 
	  uint8_t  tx_buf[2];
	  uint8_t  rx_buf[3];
	  //------------------------------------------------------------------
    tx_buf[0] = 0x82;
	  m_transfer_completed = false;
    nrf_drv_spi_transfer(&m_spi_master_0, tx_buf, 1, rx_buf, 3);
	  while(m_transfer_completed==false);
	  //------------------------------------------------------------------
	  // Merge high byte (8b) and low bits (2b) into 16b signed destination
    m_g->x_axis  = ( (((uint8_t)rx_buf[1]) >> 6) | ((uint16_t)(rx_buf[2]) << 2) );
	  m_g->x_axis &= 0x3FF;
	  if( m_g->x_axis > 511 ) m_g->x_axis -= 1023;
	
	  //Low Pass Filter
	  m_g->fXg = m_g->x_axis * 0.1 + (m_g->fXg * 0.9);
	  m_g->x_axis=(int16_t)m_g->fXg;
}

void ReadG_ID(m_bma250e_t* m_g)
{ 
	  uint8_t  tx_buf[2];
	  uint8_t  rx_buf[2];
	  //------------------------------------------------------------------
    tx_buf[0] = 0x80;
	  m_transfer_completed = false;
    nrf_drv_spi_transfer(&m_spi_master_0, tx_buf, 1, rx_buf, 2);
	  while(m_transfer_completed==false);
	  //------------------------------------------------------------------
	  m_g->ID  = rx_buf[1];
	  tx_buf[0] = 0x80;
}


