/*

Speed   Sensor Model Name: TMS10
Cadence Sensor Model Name: TMC10
Combo   Sensor Model Name: TSC10

BLE CSC and ANT CSC TX

*/
#include <stdint.h>
#include <string.h>
#include "nordic_common.h"
#include "nrf.h"
#include "app_error.h"
#include "nrf_gpio.h"
#include "nrf.h"
#include "ble.h"
#include "ble_hci.h"
#include "ble_srv_common.h"
#include "ble_advdata.h"
#include "ble_bas.h"
#include "ble_cscs.h"
#include "ble_sc_ctrlpt.h"
#include "ble_dis.h"
#include "ble_conn_params.h"
#include "boards.h"
#include "softdevice_handler.h"
#include "app_timer.h"
#include "device_manager.h"
#include "ant_parameters.h"
#include "ant_interface.h"
#include "pstorage.h"
#include "app_trace.h"
#include "bsp.h"
#include "ant_error.h"
#include "ant_stack_config.h"
#include "ant_bsc.h"

#include "app_scheduler.h"

#include "nrf_delay.h"
#include "app_uart.h"
#include "nrf_drv_uart.h"
#include "uicr_config.h"
#include "m_led.h"
#include "m_wakeup_btn.h"
#include "m_adc.h"
#include "m_csc.h"
#include "m_bma250e.h"

#include "fstorage.h"
FS_SECTION_VARS_ADD( fs_config_t fs_config ) = { .cb = &fs_callback, .num_pages = 1, .page_order = 1 };


#define DEVICE_NAME                     "SPD"                                        /**< Name of device. Will be included in the advertising data. */
#define MANUFACTURER_NAME               "TOPACTION"                                  /**< Manufacturer. Will be passed to Device Information Service. */
#define APP_ADV_INTERVAL                40                                           /**< The advertising interval (in units of 0.625 ms. This value corresponds to 25 ms). */
#define APP_ADV_TIMEOUT_IN_SECONDS      180                                          /**< The advertising timeout in units of seconds. */

#define IS_SRVC_CHANGED_CHARACT_PRESENT 0                                            /**< Whether or not to include the service_changed characteristic. If not enabled, the server's database cannot be changed for the lifetime of the device */
#define APP_TIMER_PRESCALER             0                                            /**< Value of the RTC1 PRESCALER register. */
#define APP_TIMER_OP_QUEUE_SIZE         10                                           /**< Size of timer operation queues. */
#define BATTERY_LEVEL_MEAS_INTERVAL     APP_TIMER_TICKS(2000, APP_TIMER_PRESCALER)   /**< Battery level measurement interval (ticks). */
#define CSC_MEAS_INTERVAL               APP_TIMER_TICKS(1000, APP_TIMER_PRESCALER)   /**< Speed and cadence measurement interval (milliseconds). */
#define CSC_EVENT_INTERVAL              APP_TIMER_TICKS(1,    APP_TIMER_PRESCALER)   
#define CSC_TIMEOUT_INTERVAL            APP_TIMER_TICKS(1000, APP_TIMER_PRESCALER)
#define BMA250E_MEAS_INTERVAL           APP_TIMER_TICKS(10,   APP_TIMER_PRESCALER)

#define SECOND_1_25_MS_UNITS            800                                          /**< Definition of 1 second, when 1 unit is 1.25 ms. */
#define SECOND_10_MS_UNITS              100                                          /**< Definition of 1 second, when 1 unit is 10 ms. */
#define MIN_CONN_INTERVAL               (SECOND_1_25_MS_UNITS / 2)                   /**< Minimum acceptable connection interval (0.5 seconds), Connection interval uses 1.25 ms units. */
#define MAX_CONN_INTERVAL               (SECOND_1_25_MS_UNITS)                       /**< Maximum acceptable connection interval (1 second), Connection interval uses 1.25 ms units. */
#define SLAVE_LATENCY                   0                                            /**< Slave latency. */
#define CONN_SUP_TIMEOUT                (4 * SECOND_10_MS_UNITS)                     /**< Connection supervisory timeout (4 seconds), Supervision Timeout uses 10 ms units. */

#define FIRST_CONN_PARAMS_UPDATE_DELAY  APP_TIMER_TICKS(5000, APP_TIMER_PRESCALER)   /**< Time from initiating event (connect or start of notification) to first time sd_ble_gap_conn_param_update is called (5 seconds). */
#define NEXT_CONN_PARAMS_UPDATE_DELAY   APP_TIMER_TICKS(30000, APP_TIMER_PRESCALER)  /**< Time between each call to sd_ble_gap_conn_param_update after the first call (30 seconds). */
#define MAX_CONN_PARAMS_UPDATE_COUNT    3                                            /**< Number of attempts before giving up the connection parameter negotiation. */

#define SEC_PARAM_TIMEOUT               30                                           /**< Timeout for Pairing Request or Security Request (in seconds). */
#define SEC_PARAM_BOND                  1                                            /**< Perform bonding. */
#define SEC_PARAM_MITM                  0                                            /**< Man In The Middle protection not required. */
#define SEC_PARAM_IO_CAPABILITIES       BLE_GAP_IO_CAPS_NONE                         /**< No I/O capabilities. */
#define SEC_PARAM_OOB                   0                                            /**< Out Of Band data not available. */
#define SEC_PARAM_MIN_KEY_SIZE          7                                            /**< Minimum encryption key size. */
#define SEC_PARAM_MAX_KEY_SIZE          16                                           /**< Maximum encryption key size. */

//#define SCHED_MAX_EVENT_DATA_SIZE       sizeof(app_timer_event_t)                   /**< Maximum size of scheduler events. Note that scheduler BLE stack events do not contain any data, as the events are being pulled from the stack in the event handler. */
//#define SCHED_QUEUE_SIZE                10                                          /**< Maximum number of events in the scheduler queue. */




#define DEAD_BEEF                       0xDEADBEEF                                   /**< Value used as error code on stack dump, can be used to identify stack location on stack unwind. */

static uint16_t                         m_conn_handle = BLE_CONN_HANDLE_INVALID;     /**< Handle of the current connection. */
static ble_gap_adv_params_t             m_adv_params;                                /**< Parameters to be passed to the stack when starting advertising. */
static ble_bas_t                        m_bas;                                       /**< Structure used to identify the battery service. */
static ble_cscs_t                       m_cscs;                                      /**< Structure used to identify the cycling speed and cadence service. */

#ifdef BONDING_ENABLE
static dm_application_instance_t        m_app_handle;                                /**< Application identifier allocated by device manager */
static bool                             m_app_initialized   = false;                 /**< Application initialized flag. */
#endif // BONDING_ENABLE

APP_TIMER_DEF(m_battery_timer_id);
APP_TIMER_DEF(m_csc_meas_timer_id);
APP_TIMER_DEF(m_csc_event_timer_id);
APP_TIMER_DEF(m_csc_timeout_timer_id);
APP_TIMER_DEF(m_bma205e_meas_timer_id);


static ble_sensor_location_t supported_locations[] = {BLE_SENSOR_LOCATION_FRONT_WHEEL ,
                                                      BLE_SENSOR_LOCATION_LEFT_CRANK  ,
                                                      BLE_SENSOR_LOCATION_RIGHT_CRANK ,
                                                      BLE_SENSOR_LOCATION_LEFT_PEDAL  ,
                                                      BLE_SENSOR_LOCATION_RIGHT_PEDAL ,
                                                      BLE_SENSOR_LOCATION_FRONT_HUB   ,
                                                      BLE_SENSOR_LOCATION_REAR_DROPOUT,
                                                      BLE_SENSOR_LOCATION_CHAINSTAY   ,
                                                      BLE_SENSOR_LOCATION_REAR_WHEEL  ,
                                                      BLE_SENSOR_LOCATION_REAR_HUB};          /**< supported location for the sensor location. */

uint32_t  m_cumulative_wheel_revs;                   /**< Cumulative wheel revolutions. */
bool      m_auto_calibration_in_progress;            /**< Set when an autocalibration is in progress. */

																											
																											

#define ANTPLUS_NETWORK_NUMBER   0                                         /**< Network number. */
#define ANTPLUS_RF_FREQ          0x39                                      /**< Frequency, Decimal 57 (2457 MHz). */
#define ANT_CSC_NETWORK_KEY      {0xB9,0xA5,0x21,0xFB,0xBD,0x72,0xC3,0x45} /**< The default network key used. */
//=========ANT Speed Tx=========================================================
#define ANT_SPEED_ANT_CHANNEL    0                        /**< SPEED ANT Channel. */
#define ANT_SPEED_DEVICE_TYPE    BSC_SPEED_DEVICE_TYPE    /**< Channel ID device type.indicates search for an ANT+ bike cadence sensor. */
#define ANT_SPEED_TRANS_TYPE     1                        /**< Transmission Type. */
#define ANT_SPEED_EXT_ASSIGN     0x01                     /**< ANT Ext Assign. */
#define ANT_SPEED_TOGGLE_MASK    0x80                     /**< SPD Page Toggle Bit Mask. */
//=========ANT Cadence Tx=======================================================
#define ANT_CADENCE_ANT_CHANNEL  1                        /**< cadence ANT Channel. */
#define ANT_CADENCE_DEVICE_TYPE  BSC_CADENCE_DEVICE_TYPE  /**< Channel ID device type.indicates search for an ANT+ bike cadence sensor. */
#define ANT_CADENCE_TRANS_TYPE   1                        /**< Transmission Type. */
#define ANT_CADENCE_EXT_ASSIGN   0x01                     /**< ANT Ext Assign. */
#define ANT_CADENCE_TOGGLE_MASK  0x80                     /**< CAD Page Toggle Bit Mask. */
//=========ANT Combo Tx=========================================================
#define ANT_COMBINED_ANT_CHANNEL 2                        /**< combo ANT Channel. */
#define ANT_COMBINED_DEVICE_TYPE BSC_COMBINED_DEVICE_TYPE /**< Device type reserved for ANT+ bike combined speed and cadence sensor */
#define ANT_COMBINED_TRANS_TYPE  1                        /**< Transmission Type. */
#define ANT_COMBINED_EXT_ASSIGN  0x01                     /**< ANT Ext Assign. */
#define ANT_COMBINED_TOGGLE_MASK 0x80                     /**< CMB Page Toggle Bit Mask. */


/** @snippet [ANT BSC TX Instance] */
void ant_bsc_evt_handler(ant_bsc_profile_t * p_profile, ant_bsc_evt_t event);
static uint8_t m_ant_network_key[] = ANT_CSC_NETWORK_KEY; /**< ANT PLUS network key. */

BSC_SENS_CHANNEL_CONFIG_DEF(m_ant_bsc,
                            ANT_SPEED_ANT_CHANNEL,
                            ANT_SPEED_TRANS_TYPE,
                            ANT_SPEED_DEVICE_TYPE,
                            UICR_ADDR_0x84,//ANT_SPEED_DEVICE_NUMBER,
                            ANTPLUS_NETWORK_NUMBER);
														
BSC_SENS_PROFILE_CONFIG_DEF(m_ant_bsc,
                            true,
                            true,
                            ANT_BSC_PAGE_5,
                            ant_bsc_evt_handler);

static ant_bsc_profile_t m_ant_bsc;

//=======Val Define===================================================================												
//uint32_t       UICR_ADDR_0x80;					
m_flash_t   m_datas;					
m_csc_t     m_csc_obj;
m_count_t   m_cnt_obj;
m_bma250e_t m_gs_obj;
														
														
														
														
uint8_t erase_flag=0;
uint8_t store_flag=0;
uint32_t m_data[10];
														
														
														
														
void timers_init(void);
void application_timers_start(void);
void application_timers_stop (void);												
									
//=====================================================================================
														
void assert_nrf_callback(uint16_t line_num, const uint8_t * p_file_name)
{
    app_error_handler(DEAD_BEEF, line_num, p_file_name);
}

//=====================================================================================

void flash_word_read(uint32_t* address, uint32_t* value)
{
    *value = *address;
}

void advertising_start(void)
{
    uint32_t err_code;
    
    err_code = sd_ble_gap_adv_start(&m_adv_params);
    if (err_code != NRF_SUCCESS && err_code != NRF_ERROR_INVALID_STATE)
    {
        APP_ERROR_HANDLER(err_code);
    }
}

void advertising_stop (void)
{
	uint32_t err_code;
  err_code = sd_ble_gap_adv_stop();
	if (err_code != NRF_SUCCESS && err_code != NRF_ERROR_INVALID_STATE)
    {
        APP_ERROR_HANDLER(err_code);
    }
}


void sleep_mode_enter(void)
{
    uint32_t   err_code;
	  ret_code_t retval;
	  //advertising_stop();
	  //on_ant_evt_channel_closed();
	  //application_timers_stop();  
	
	  erase_flag=1;
		retval =fs_erase(&fs_config, fs_config.p_start_addr,0x100);
	  if (retval != NRF_SUCCESS)
    {
        //return retval;
    }
		while(erase_flag==1);
    
		store_flag=1;
	  m_data[9] = 5566;
	  retval = fs_store(&fs_config, fs_config.p_start_addr, &m_data[9], 10);
		if (retval != NRF_SUCCESS)
    {
        //return retval;
    }
		while(store_flag==1);
	
	
    low_power_stop     ();//Close LED PWM
    wakeup_gpio_init   ();
    BMA250E_Wakeup_Mode();
	  // Go to system-off mode (this function will not return; wakeup will cause a reset).
    err_code = sd_power_system_off();
    APP_ERROR_CHECK(err_code);
}

void ant_SpeedTx_init(void)
{
    uint32_t err_code;
    
    err_code = ant_stack_static_config();
    APP_ERROR_CHECK(err_code);
	
    err_code = sd_ant_network_address_set(ANTPLUS_NETWORK_NUMBER, m_ant_network_key);
    APP_ERROR_CHECK(err_code);
    //=====================================================================================
    //Speed Init
    err_code = ant_bsc_sens_init(&m_ant_bsc, BSC_SENS_CHANNEL_CONFIG(m_ant_bsc), BSC_SENS_PROFILE_CONFIG(m_ant_bsc));
    APP_ERROR_CHECK(err_code);
		
		//Here we set the ant device number again, because default number is fix
	  err_code = sd_ant_channel_id_set(ANT_SPEED_ANT_CHANNEL, UICR_ADDR_0x80, ANT_SPEED_DEVICE_TYPE, ANT_SPEED_TRANS_TYPE);
	  APP_ERROR_CHECK(err_code);
	
    m_ant_bsc.BSC_PROFILE_manuf_id     = UICR_ADDR_0x90;//BSC_MF_ID;
    m_ant_bsc.BSC_PROFILE_serial_num   = UICR_ADDR_0x80;//BSC_SERIAL_NUMBER;
    m_ant_bsc.BSC_PROFILE_hw_version   = UICR_ADDR_0x88;//BSC_HW_VERSION;
    m_ant_bsc.BSC_PROFILE_sw_version   = UICR_ADDR_0x84;//BSC_SW_VERSION;
    m_ant_bsc.BSC_PROFILE_model_num    = UICR_ADDR_0x8C;//BSC_MODEL_NUMBER;
    err_code = ant_bsc_sens_open(&m_ant_bsc);
    APP_ERROR_CHECK(err_code);
}


void ant_bsc_evt_handler(ant_bsc_profile_t * p_profile, ant_bsc_evt_t event)
{
    switch (event)
    {
        case ANT_BSC_PAGE_0_UPDATED:
            /* fall through */
        case ANT_BSC_PAGE_1_UPDATED:
            /* fall through */
        case ANT_BSC_PAGE_2_UPDATED:
            /* fall through */
        case ANT_BSC_PAGE_3_UPDATED:
            /* fall through */
        case ANT_BSC_PAGE_4_UPDATED:
            /* fall through */
        case ANT_BSC_PAGE_5_UPDATED:
            /* fall through */
        case ANT_BSC_COMB_PAGE_0_UPDATED:
		        //ant_bsc_simulator_one_iteration(&m_ant_bsc_simulator);
            break;

        default:
            break;
    }
}

static void on_ant_evt(ant_evt_t * p_ant_evt)
{
	if (p_ant_evt->channel == ANT_SPEED_ANT_CHANNEL)//Speed
    {
        switch (p_ant_evt->event)
        {
            case EVENT_TX:
                m_ant_bsc.page_0.event_time = m_csc_obj.speed_event_time;
                m_ant_bsc.page_0.rev_count  = m_csc_obj.ant_speed_revolution_cnt;
                ant_bsc_sens_evt_handler(&m_ant_bsc, p_ant_evt);
                break;
					
            case EVENT_CHANNEL_CLOSED:
                //Ant TX won't timeout and close channel
                //on_ant_evt_channel_closed();
                break;

            default:
                // No implementation needed.
                break;
        }
    }
}


uint8_t MyBattery_anlysis (int32_t ad_data)
{
	uint8_t batt=0;
	float   batt_vaule  =0.0;
	static  uint8_t flag=0;
	
	batt_vaule = ad_data;
  batt_vaule = (batt_vaule*BATTERY)-0.01;
  //bat_status: 1-New,2-Good,3-ok,4-Low,5-Critical
  //coarse_bat_volt xx.
  //fract_bat_volt    .xxx
	if( flag == 1 ) {	
	  if( batt_vaule >= 2.83 ){
		  batt=100;
      m_ant_bsc.page_4.bat_status=1;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=255;
	  }else if( batt_vaule >= 2.73 ){
		  batt=75;
      m_ant_bsc.page_4.bat_status=2;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=211;
	  }else if( batt_vaule >= 2.65 ){
		  batt=50;
      m_ant_bsc.page_4.bat_status=2;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=186;
	  }else if( batt_vaule >= 2.59 ){
		  batt=25;
      m_ant_bsc.page_4.bat_status=3;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=165;
	  }else if( batt_vaule < 2.73 ){
		  batt=75;
      m_ant_bsc.page_4.bat_status=2;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=186;
	  }else if( batt_vaule < 2.65 ){
		  batt=50;
      m_ant_bsc.page_4.bat_status=3;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=165;
	  }else if( batt_vaule < 2.59 ){
		  batt=25;
      m_ant_bsc.page_4.bat_status=4;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=150;
	  }else if( batt_vaule < 2.49 ){
		  batt=5;
      m_ant_bsc.page_4.bat_status=5;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=125;
	  }
  }else{
		if( batt_vaule >= 2.73 ){
		  batt=100;
      m_ant_bsc.page_4.bat_status=1;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=255;
	  }else if( batt_vaule < 2.73 ){
			flag=1;
		  batt=75;
      m_ant_bsc.page_4.bat_status=2;
      m_ant_bsc.page_4.coarse_bat_volt=2;
      m_ant_bsc.page_4.fract_bat_volt=211;
	  }
	}
	
	return batt;
}

static void battery_level_update(void)
{
    uint32_t err_code;
    uint8_t  battery_level;

	  nrf_adc_start();// trigger next ADC conversion
    nrf_delay_ms(1);
    battery_level = MyBattery_anlysis(adc_sample);
	  
	  //if( m_ant_bsc.page_1.operating_time < 0x00FFFFFF) {//The operating time increments by one count every two seconds
	  //    m_ant_bsc.page_1.operating_time++;
		//}else{
		//	  m_ant_bsc.page_1.operating_time=0;
		//}
		
    err_code = ble_bas_battery_level_update(&m_bas, battery_level);
    if ((err_code != NRF_SUCCESS) &&
        (err_code != NRF_ERROR_INVALID_STATE) &&
        (err_code != BLE_ERROR_NO_TX_BUFFERS) &&
        (err_code != BLE_ERROR_GATTS_SYS_ATTR_MISSING)
        )
    {
        APP_ERROR_HANDLER(err_code);
    }
}

static void battery_level_meas_timeout_handler(void * p_context)
{
    UNUSED_PARAMETER(p_context);
    battery_level_update();
}

static void csc_sim_measurement(ble_cscs_meas_t * p_measurement)
{ 
    // Calculate simulated wheel revolution values.
    p_measurement->is_wheel_rev_data_present = true;
    p_measurement->cumulative_wheel_revs     = m_csc_obj.ble_speed_revolution_cnt;
    p_measurement->last_wheel_event_time     = m_csc_obj.speed_event_time;
	  // Calculate simulated cadence values.
    p_measurement->is_crank_rev_data_present = false;
    p_measurement->cumulative_crank_revs     = 0;
    p_measurement->last_crank_event_time     = 0;
}

static void csc_meas_timeout_handler(void * p_context)
{
    uint32_t        err_code;
    ble_cscs_meas_t cscs_measurement;

    UNUSED_PARAMETER(p_context);

    csc_sim_measurement(&cscs_measurement);

    err_code = ble_cscs_measurement_send(&m_cscs, &cscs_measurement);
    if ((err_code != NRF_SUCCESS) &&
        (err_code != NRF_ERROR_INVALID_STATE) &&
        (err_code != BLE_ERROR_NO_TX_BUFFERS) &&
        (err_code != BLE_ERROR_GATTS_SYS_ATTR_MISSING)
        )
    {
        APP_ERROR_HANDLER(err_code);
    }
		
    if (m_auto_calibration_in_progress)
    {
        err_code = ble_sc_ctrlpt_rsp_send(&(m_cscs.ctrl_pt), BLE_SCPT_SUCCESS);
        if ((err_code != NRF_SUCCESS) &&
            (err_code != NRF_ERROR_INVALID_STATE) &&
            (err_code != BLE_ERROR_NO_TX_BUFFERS)
            )
        {
            APP_ERROR_HANDLER(err_code);
        }
        if (err_code != BLE_ERROR_NO_TX_BUFFERS)
        {
            m_auto_calibration_in_progress = false;
        }
    }

}

/**@brief Function for handling Speed and Cadence Control point events
 *
 * @details Function for handling Speed and Cadence Control point events.
 *          This function parses the event and in case the "set cumulative value" event is received,
 *          sets the wheel cumulative value to the received value.
 */
ble_scpt_response_t sc_ctrlpt_event_handler(ble_sc_ctrlpt_t *p_sc_ctrlpt, ble_sc_ctrlpt_evt_t * p_evt)
{
    switch (p_evt->evt_type)
    {
        case BLE_SC_CTRLPT_EVT_SET_CUMUL_VALUE:
            m_cumulative_wheel_revs = p_evt->params.cumulative_value;
            break;

        case BLE_SC_CTRLPT_EVT_START_CALIBRATION:
            m_auto_calibration_in_progress = true;
            break;

        default:
            // No implementation needed.
            break;
    }
    return (BLE_SCPT_SUCCESS);
}

static void csc_event_timeout_handler(void * p_context)
{
    UNUSED_PARAMETER(p_context);
	
	
}

static void csc_timeout_handler(void * p_context)
{
    UNUSED_PARAMETER(p_context);
	  count_sleep_time(&m_cnt_obj);
	  count_stop_time (&m_cnt_obj);
}

static void bma250e_meas_timeout_handler(void * p_context)
{
    UNUSED_PARAMETER(p_context);
	  ReadAccX16(&m_gs_obj);
}

void timers_init(void)
{
	  uint32_t err_code;
    // Initialize timer module
    APP_TIMER_INIT(APP_TIMER_PRESCALER, APP_TIMER_OP_QUEUE_SIZE, NULL);
    err_code = app_timer_create(&m_battery_timer_id,   APP_TIMER_MODE_REPEATED,battery_level_meas_timeout_handler);
    APP_ERROR_CHECK(err_code);
    err_code = app_timer_create(&m_csc_meas_timer_id,  APP_TIMER_MODE_REPEATED,csc_meas_timeout_handler);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_create(&m_csc_event_timer_id, APP_TIMER_MODE_REPEATED,csc_event_timeout_handler);
    APP_ERROR_CHECK(err_code);
    err_code = app_timer_create(&m_csc_timeout_timer_id,APP_TIMER_MODE_REPEATED,csc_timeout_handler);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_create(&m_bma205e_meas_timer_id,APP_TIMER_MODE_REPEATED,bma250e_meas_timeout_handler);
    APP_ERROR_CHECK(err_code);
}

void application_timers_start(void)
{
    uint32_t err_code;
    err_code = app_timer_start(m_battery_timer_id,     BATTERY_LEVEL_MEAS_INTERVAL, NULL);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_start(m_csc_meas_timer_id,    CSC_MEAS_INTERVAL, NULL);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_start(m_csc_event_timer_id,   CSC_EVENT_INTERVAL, NULL);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_start(m_csc_timeout_timer_id, CSC_TIMEOUT_INTERVAL, NULL);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_start(m_bma205e_meas_timer_id, BMA250E_MEAS_INTERVAL, NULL);
    APP_ERROR_CHECK(err_code);
}

void application_timers_stop(void)
{
    uint32_t err_code;
    err_code = app_timer_stop(m_battery_timer_id);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_stop(m_csc_meas_timer_id);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_stop(m_csc_event_timer_id);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_stop(m_csc_timeout_timer_id);
    APP_ERROR_CHECK(err_code);
	  err_code = app_timer_stop(m_bma205e_meas_timer_id);
    APP_ERROR_CHECK(err_code);
}

/**@brief GAP initialization.
 *
 * @details This function sets up all the necessary GAP (Generic Access Profile) parameters of the
 *          device including the device name, appearance, and the preferred connection parameters.
 */
static void gap_params_init(void)
{
    uint32_t                err_code;
    ble_gap_conn_params_t   gap_conn_params;
    ble_gap_conn_sec_mode_t sec_mode;
    char                    dev_name[100];
   
	
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&sec_mode);
	  
	  sprintf(dev_name,"SPD10-%05d",UICR_ADDR_0x80);
    //sprintf(dev_name,"TMS10-%05d",UICR_ADDR_0x80);
	  //sprintf(dev_name,"BCP-65-%05d",UICR_ADDR_0x80);
	  
	  err_code = sd_ble_gap_device_name_set(&sec_mode,(const uint8_t *)dev_name,strlen(dev_name));
	  APP_ERROR_CHECK(err_code);

	  err_code = sd_ble_gap_appearance_set(BLE_APPEARANCE_CYCLING_SPEED_SENSOR);//SPEED
    APP_ERROR_CHECK(err_code);
    
    memset(&gap_conn_params, 0, sizeof(gap_conn_params));

    gap_conn_params.min_conn_interval = MIN_CONN_INTERVAL;
    gap_conn_params.max_conn_interval = MAX_CONN_INTERVAL;
    gap_conn_params.slave_latency     = SLAVE_LATENCY;
    gap_conn_params.conn_sup_timeout  = CONN_SUP_TIMEOUT;

    err_code = sd_ble_gap_ppcp_set(&gap_conn_params);
    APP_ERROR_CHECK(err_code);
}




/**@brief Advertising functionality initialization.
 *
 * @details Encodes the required advertising data and passes it to the stack.
 *          Also builds a structure to be passed to the stack when starting advertising.
 */
static void advertising_init(void)
{
    uint32_t      err_code;
    ble_advdata_t advdata;
    uint8_t       flags = BLE_GAP_ADV_FLAGS_LE_ONLY_GENERAL_DISC_MODE;
    
    ble_uuid_t adv_uuids[] =
    {
        {BLE_UUID_CYCLING_SPEED_AND_CADENCE,  BLE_UUID_TYPE_BLE},
        {BLE_UUID_BATTERY_SERVICE,            BLE_UUID_TYPE_BLE},
        {BLE_UUID_DEVICE_INFORMATION_SERVICE, BLE_UUID_TYPE_BLE}
    };
    // Build and set advertising data.
    memset(&advdata, 0, sizeof(advdata));
    
    advdata.name_type               = BLE_ADVDATA_FULL_NAME;
    advdata.include_appearance      = true;
    advdata.flags                   = flags;
    advdata.uuids_complete.uuid_cnt = sizeof(adv_uuids) / sizeof(adv_uuids[0]);
    advdata.uuids_complete.p_uuids  = adv_uuids;
    
    err_code = ble_advdata_set(&advdata, NULL);
    APP_ERROR_CHECK(err_code);

    // Initialise advertising parameters (used when starting advertising).
    memset(&m_adv_params, 0, sizeof(m_adv_params));
    
    m_adv_params.type        = BLE_GAP_ADV_TYPE_ADV_IND;
    m_adv_params.p_peer_addr = NULL;
    m_adv_params.fp          = BLE_GAP_ADV_FP_ANY;
    m_adv_params.interval    = APP_ADV_INTERVAL;
    m_adv_params.timeout     = APP_ADV_TIMEOUT_IN_SECONDS;
}

/**@brief Initialize services that will be used by the application.
 *
 * @details Initialize the Heart Rate and Device Information services.
 */
static void services_init(void)
{
    uint32_t              err_code;
    ble_cscs_init_t       cscs_init;
    ble_bas_init_t        bas_init;
    ble_dis_init_t        dis_init;
    ble_sensor_location_t sensor_location;
    char tmp0[100];
    char tmp1[100];
    char tmp2[100];
    char tmp3[100];
	
    // Initialize Cycling Speed and Cadence Service.
    memset(&cscs_init, 0, sizeof(cscs_init));

    cscs_init.evt_handler = NULL;
    cscs_init.feature     = BLE_CSCS_FEATURE_WHEEL_REV_BIT;

    // Here the sec level for the Cycling Speed and Cadence Service can be changed/increased.
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cscs_init.csc_meas_attr_md.cccd_write_perm);   // for the measurement characteristic, only the CCCD write permission can be set by the application, others are mandated by service specification
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cscs_init.csc_feature_attr_md.read_perm);      // for the feature characteristic, only the read permission can be set by the application, others are mandated by service specification
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cscs_init.csc_ctrlpt_attr_md.write_perm);      // for the SC control point characteristic, only the write permission and CCCD write can be set by the application, others are mandated by service specification
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cscs_init.csc_ctrlpt_attr_md.cccd_write_perm); // for the SC control point characteristic, only the write permission and CCCD write can be set by the application, others are mandated by service specification
    
	  //2015/12/8,control point need modify later
    cscs_init.ctrplt_supported_functions = BLE_SRV_SC_CTRLPT_CUM_VAL_OP_SUPPORTED|BLE_SRV_SC_CTRLPT_SENSOR_LOCATIONS_OP_SUPPORTED|BLE_SRV_SC_CTRLPT_START_CALIB_OP_SUPPORTED;
    cscs_init.ctrlpt_evt_handler            = sc_ctrlpt_event_handler;
		cscs_init.list_supported_locations      = supported_locations;
    cscs_init.size_list_supported_locations = sizeof(supported_locations) / sizeof(ble_sensor_location_t);            
    
    sensor_location           = BLE_SENSOR_LOCATION_FRONT_WHEEL;                    // initializes the sensor location to add the sensor location characteristic.
		cscs_init.sensor_location = &sensor_location;
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&cscs_init.csc_sensor_loc_attr_md.read_perm);    // for the sensor location characteristic, only the read permission can be set by the application, others are mendated by service specification

    err_code = ble_cscs_init(&m_cscs, &cscs_init);
    APP_ERROR_CHECK(err_code);

    // Initialize Battery Service.
    memset(&bas_init, 0, sizeof(bas_init));
    // Here the sec level for the Battery Service can be changed/increased.
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&bas_init.battery_level_char_attr_md.cccd_write_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&bas_init.battery_level_char_attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&bas_init.battery_level_char_attr_md.write_perm);
    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&bas_init.battery_level_report_read_perm);

    bas_init.evt_handler          = NULL;
    bas_init.support_notification = true;
    bas_init.p_report_ref         = NULL;
    bas_init.initial_batt_level   = 100;

    err_code = ble_bas_init(&m_bas, &bas_init);
    APP_ERROR_CHECK(err_code);

    // Initialize Device Information Service.
    memset(&dis_init, 0, sizeof(dis_init));
    ble_srv_ascii_to_utf8(&dis_init.manufact_name_str, MANUFACTURER_NAME);
		
    sprintf(tmp0,"%d",UICR_ADDR_0x80);
    ble_srv_ascii_to_utf8(&dis_init.serial_num_str, tmp0);
    sprintf(tmp1,"%d",UICR_ADDR_0x88);
    ble_srv_ascii_to_utf8(&dis_init.hw_rev_str, tmp1);
    sprintf(tmp2,"%d",UICR_ADDR_0x84);
    ble_srv_ascii_to_utf8(&dis_init.sw_rev_str, tmp2);
    sprintf(tmp3,"%d",UICR_ADDR_0x8C);
    ble_srv_ascii_to_utf8(&dis_init.model_num_str, tmp3);

    BLE_GAP_CONN_SEC_MODE_SET_OPEN(&dis_init.dis_attr_md.read_perm);
    BLE_GAP_CONN_SEC_MODE_SET_NO_ACCESS(&dis_init.dis_attr_md.write_perm);

    err_code = ble_dis_init(&dis_init);
    APP_ERROR_CHECK(err_code);
}

/**@brief Connection Parameters Module handler.
 *
 * @details This function will be called for all events in the Connection Parameters Module which
 *          are passed to the application.
 *          @note All this function does is to disconnect. This could have been done by simply
 *                setting the disconnect_on_fail config parameter, but instead we use the event
 *                handler mechanism to demonstrate its use.
 *
 * @param[in]   p_evt   Event received from the Connection Parameters Module.
 */
static void on_conn_params_evt(ble_conn_params_evt_t * p_evt)
{
    uint32_t err_code;
    
    switch (p_evt->evt_type)
    {
        case BLE_CONN_PARAMS_EVT_FAILED:
            err_code = sd_ble_gap_disconnect(m_conn_handle, BLE_HCI_CONN_INTERVAL_UNACCEPTABLE);
            APP_ERROR_CHECK(err_code);
            break;
            
        default:
            // No implementation needed.
            break;
    }
}

/**@brief Connection Parameters module error handler.
 *
 * @param[in]   nrf_error   Error code containing information about what went wrong.
 */
static void conn_params_error_handler(uint32_t nrf_error)
{
    APP_ERROR_HANDLER(nrf_error);
}

/**@brief Initialize the Connection Parameters module.
 */
static void conn_params_init(void)
{
    uint32_t               err_code;
    ble_conn_params_init_t cp_init;

    memset(&cp_init, 0, sizeof(cp_init));

    cp_init.p_conn_params                  = NULL;
    cp_init.first_conn_params_update_delay = FIRST_CONN_PARAMS_UPDATE_DELAY;
    cp_init.next_conn_params_update_delay  = NEXT_CONN_PARAMS_UPDATE_DELAY;
    cp_init.max_conn_params_update_count   = MAX_CONN_PARAMS_UPDATE_COUNT;
    cp_init.start_on_notify_cccd_handle    = m_cscs.meas_handles.cccd_handle;
    cp_init.disconnect_on_fail             = false;
    cp_init.evt_handler                    = on_conn_params_evt;
    cp_init.error_handler                  = conn_params_error_handler;
    
    err_code = ble_conn_params_init(&cp_init);
    APP_ERROR_CHECK(err_code);
}

/**@brief Application's Stack BLE event handler.
 *
 * @param[in]   p_ble_evt   Bluetooth stack event.
 */
static void on_ble_evt(ble_evt_t * p_ble_evt)
{
    uint32_t err_code = NRF_SUCCESS;

    switch (p_ble_evt->header.evt_id)
    {
        case BLE_GAP_EVT_CONNECTED:
            m_conn_handle = p_ble_evt->evt.gap_evt.conn_handle;
            break;
            
        case BLE_GAP_EVT_DISCONNECTED:
				    advertising_start();
            // Note: Bonding information will be stored, advertising will be restarted and the
            //       ANT channel will be reopened when ANT event CHANNEL_CLOSED is received.
            break;

#ifndef BONDING_ENABLE
        case BLE_GAP_EVT_SEC_PARAMS_REQUEST:
            err_code = sd_ble_gap_sec_params_reply(m_conn_handle, 
                                                   BLE_GAP_SEC_STATUS_PAIRING_NOT_SUPP, 
                                                   NULL,
                                                   NULL);
            APP_ERROR_CHECK(err_code);
            break;
#endif // BONDING_ENABLE

        case BLE_GAP_EVT_TIMEOUT:
            if (p_ble_evt->evt.gap_evt.params.timeout.src == BLE_GAP_TIMEOUT_SRC_ADVERTISING)
            { 
							  #if 1
							  advertising_start();//even timeout keep advertising
							  #else 
							  sleep_mode_enter();
							  #endif
            }
            break;

#ifndef	BONDING_ENABLE
            case BLE_GATTS_EVT_SYS_ATTR_MISSING:
                err_code = sd_ble_gatts_sys_attr_set(m_conn_handle,
                                                     NULL,
                                                     0,
                                                     BLE_GATTS_SYS_ATTR_FLAG_SYS_SRVCS | BLE_GATTS_SYS_ATTR_FLAG_USR_SRVCS);
                APP_ERROR_CHECK(err_code);
                break;
#endif // BONDING_ENABLE
            
        default:
            // No implementation needed.
            break;
    }
}


/**@brief Dispatches a stack event to all modules with a stack BLE event handler.
 *
 * @details This function is called from the Stack event interrupt handler after a stack BLE
 *          event has been received.
 *
 * @param[in]   p_ble_evt   Stack Bluetooth event.
 */
static void ble_evt_dispatch(ble_evt_t * p_ble_evt)
{
#ifdef BONDING_ENABLE
    dm_ble_evt_handler(p_ble_evt);
#endif // BONDING_ENABLE

    ble_cscs_on_ble_evt(&m_cscs, p_ble_evt);
    ble_bas_on_ble_evt (&m_bas, p_ble_evt);
    ble_conn_params_on_ble_evt(p_ble_evt);
    on_ble_evt(p_ble_evt);
}



static void sys_evt_dispatch(uint32_t sys_evt)
{
    
    fs_sys_event_handler(sys_evt);
	  pstorage_sys_event_handler(sys_evt);
}


#ifdef BONDING_ENABLE 
/**@brief Function for dispatching a system event to interested modules.
 *
 * @details This function is called from the System event interrupt handler after a system
 *          event has been received.
 *
 * @param[in]   sys_evt   System stack event.
 */
static void sys_evt_dispatch(uint32_t sys_evt)
{
    uint32_t err_code;
    uint32_t count;

    pstorage_sys_event_handler(sys_evt);

    // Verify if BLE bond data storage access is in progress or not.    
    if (m_app_initialized == true)
    {
        err_code = pstorage_access_status_get(&count);
        if ((err_code == NRF_SUCCESS) && (count == 0))
        {        
            advertising_start();
        }
    }
    else
    {
        m_app_initialized = true;
    }
}


/**@brief Function for handling the Device Manager events.
 *
 * @param[in]   p_evt   Data associated to the device manager event.
 */
static uint32_t device_manager_evt_handler(dm_handle_t const    * p_handle,
                                           dm_event_t const     * p_event,
                                           ret_code_t             event_result)
{
    APP_ERROR_CHECK(event_result);
    return NRF_SUCCESS;
}


/**@brief Function for the Device Manager initialization.
 */
static void device_manager_init(void)
{
    uint32_t                err_code;
    dm_init_param_t         init_data;
    dm_application_param_t  register_param;
    
    // Initialize persistent storage module.
    err_code = pstorage_init();
    APP_ERROR_CHECK(err_code);

    err_code = dm_init(&init_data);
    APP_ERROR_CHECK(err_code);

    memset(&register_param.sec_param, 0, sizeof(ble_gap_sec_params_t));
    //register_param.sec_param.timeout      = SEC_PARAM_TIMEOUT;
    register_param.sec_param.bond         = SEC_PARAM_BOND;
    register_param.sec_param.mitm         = SEC_PARAM_MITM;
    register_param.sec_param.io_caps      = SEC_PARAM_IO_CAPABILITIES;
    register_param.sec_param.oob          = SEC_PARAM_OOB;
    register_param.sec_param.min_key_size = SEC_PARAM_MIN_KEY_SIZE;
    register_param.sec_param.max_key_size = SEC_PARAM_MAX_KEY_SIZE;
    register_param.evt_handler            = device_manager_evt_handler;
    register_param.service_type           = DM_PROTOCOL_CNTXT_GATT_SRVR_ID;

    err_code = dm_register(&m_app_handle, &register_param);
    APP_ERROR_CHECK(err_code);
}
#endif // BONDING_ENABLE


/**@brief BLE + ANT stack initialization.
 *
 * @details Initializes the SoftDevice and the stack event interrupt.
 */
static void ble_ant_stack_init(void)
{
    uint32_t                err_code;

    // Initialize SoftDevice
    SOFTDEVICE_HANDLER_INIT(NRF_CLOCK_LFCLKSRC_SYNTH_250_PPM, NULL);
    
    // Initialize BLE stack
    ble_enable_params_t ble_enable_params;
    memset(&ble_enable_params, 0, sizeof(ble_enable_params));
    ble_enable_params.gatts_enable_params.service_changed = IS_SRVC_CHANGED_CHARACT_PRESENT;
    err_code = sd_ble_enable(&ble_enable_params);
    APP_ERROR_CHECK(err_code);

    // Subscribe for BLE events.
    err_code = softdevice_ble_evt_handler_set(ble_evt_dispatch);
    APP_ERROR_CHECK(err_code);
        
    // Subscribe for ANT events.
    err_code = softdevice_ant_evt_handler_set(on_ant_evt);
    APP_ERROR_CHECK(err_code);

//#ifdef BONDING_ENABLE
    // Register with the SoftDevice handler module for BLE events.
    err_code = softdevice_sys_evt_handler_set(sys_evt_dispatch);
    APP_ERROR_CHECK(err_code);
//#endif // BONDING_ENABLE
}

static void power_manage(void)
{
    uint32_t err_code;
    err_code = sd_app_evt_wait();// Wait for events 
    APP_ERROR_CHECK(err_code);
}

#if 0
/**@brief Function for the Event Scheduler initialization.
 */
static void scheduler_init(void)
{
    APP_SCHED_INIT(SCHED_MAX_EVENT_DATA_SIZE, SCHED_QUEUE_SIZE);
}
#endif

static void fs_callback(uint8_t           op_code,
                        uint32_t          result,
                        uint32_t  const * p_data,
                        fs_length_t       length)
{
   switch(op_code){
		 case FS_OP_NONE:
			  
		 break;
     case FS_OP_STORE:
        store_flag=0;
     break;
     case FS_OP_ERASE:
        erase_flag=0;
     break;		 
	 }
}		




/**@brief Application main function.
 */
int main(void)
{
	  //uint32_t err_code;
	  memset(&m_datas,   0, sizeof(m_datas));//system use datas
	  memset(&m_csc_obj, 0, sizeof(m_csc_obj));
	  memset(&m_cnt_obj, 0, sizeof(m_cnt_obj));
	  memset(&m_gs_obj,  0, sizeof(m_gs_obj));
	 
	  //flash_word_read((uint32_t * )0x10001080, &UICR_ADDR_0x80);//Read the Device number
    //MyParameterRead (&m_datas);
    //csc_parameter_update(&m_datas, &m_csc_obj);
	  //m_ant_bsc.page_1.operating_time = m_csc_obj.ant_op_time;
	
	  
    // Initialize peripherals
    timers_init();
    low_power_init();
	  m_adc_config();
	  spi_master_init();
	  BMA250E_Runtime_Mode();
	  ReadG_ID(&m_gs_obj);//Just make sure ID:0xF9
	  
	  
	
    // Initialize S310 SoftDevice
    ble_ant_stack_init();
	
	  //scheduler_init();
    // Initialize Bluetooth stack parameters.
    gap_params_init();
    advertising_init();
    services_init();
    conn_params_init();
		
    // Initialize ANT CSC TX channel.
    ant_SpeedTx_init();
    application_timers_start();

   
	  ret_code_t retval = fs_init();
    if (retval != NRF_SUCCESS)
    {
        // fs_init() failed, propagate the error.
        return retval;
    }
		
		flash_word_read(fs_config.p_start_addr, &m_data[9]);
		m_ant_bsc.page_1.operating_time = (m_data[9]>>1)+1;
		
		#if 0
		erase_flag=1;
		retval =fs_erase(&fs_config, fs_config.p_start_addr,0x100);
	  if (retval != NRF_SUCCESS)
    {
        //return retval;
    }
		while(erase_flag==1);
    
		store_flag=1;
	  m_data[9] = 5566;
	  retval = fs_store(&fs_config, fs_config.p_start_addr, &m_data[9], 10);
		if (retval != NRF_SUCCESS)
    {
        return retval;
    }
		while(store_flag==1);
    #endif
		
		
		
#ifdef BONDING_ENABLE
    uint32_t count; 
    // Initialize device manager.
    device_manager_init();
    err_code = pstorage_access_status_get(&count);
    if ((err_code == NRF_SUCCESS) && (count == 0))
#endif // BONDING_ENABLE
    {
        advertising_start();
    }

	
    // Enter main loop.
    for (;;)
    {
			  //app_sched_execute();
        power_manage();
    }
}

/** 
 * @}
 */
